From d3e6659be4b92924e7305a6ff5163b353a027747 Mon Sep 17 00:00:00 2001
From: Yau Wai Gan <yau.wai.gan@.intel.com>
Date: Tue, 13 Oct 2020 14:38:32 +0800
Subject: [PATCH] [PATCH] spl: socfpga: Enable double peripheral RBF
 configuration

Double peripheral RBF configuration are needed on some devices or boards
to stabilize the IO configuration system. This patch is only applicable
for specific use case and devices of Arria10 SoC. Please contact Intel
Customer Support for more details.

Signed-off-by: Kallur Ramegowda, Sunil <sunil.kallur.ramegowda@intel.com>
Signed-off-by: Tien Fong Chee <tien.fong.chee@intel.com>
Signed-off-by: Yau Wai Gan <yau.wai.gan@.intel.com>
---
 .../include/mach/fpga_manager_arria10.h            |  1 +
 arch/arm/mach-socfpga/spl_a10.c                    |  3 +++
 drivers/fpga/socfpga_arria10.c                     | 28 +++++++++++++++++++++-
 3 files changed, 31 insertions(+), 1 deletion(-)

diff --git a/arch/arm/mach-socfpga/include/mach/fpga_manager_arria10.h b/arch/arm/mach-socfpga/include/mach/fpga_manager_arria10.h
index 62249b3695..e668ff34c1 100644
--- a/arch/arm/mach-socfpga/include/mach/fpga_manager_arria10.h
+++ b/arch/arm/mach-socfpga/include/mach/fpga_manager_arria10.h
@@ -130,6 +130,7 @@ int is_fpgamgr_early_user_mode(void);
 int socfpga_loadfs(fpga_fs_info *fpga_fsinfo, const void *buf, size_t bsize,
 		  u32 offset);
 void fpgamgr_program(const void *buf, size_t bsize, u32 offset);
+void fpgamgr_reprogram(const void *buf, size_t bsize, u32 offset);
 #endif /* __ASSEMBLY__ */
 
 #endif /* _FPGA_MANAGER_ARRIA10_H_ */
diff --git a/arch/arm/mach-socfpga/spl_a10.c b/arch/arm/mach-socfpga/spl_a10.c
index 2151a9b530..5e27262ff5 100644
--- a/arch/arm/mach-socfpga/spl_a10.c
+++ b/arch/arm/mach-socfpga/spl_a10.c
@@ -131,6 +131,9 @@ void spl_board_init(void)
 	} else if (!is_fpgamgr_early_user_mode()) {
 		/* Program IOSSM(early IO release) or full FPGA */
 		fpgamgr_program(buf, FPGA_BUFSIZ, 0);
+
+		/* Re-program IOSSM to stabilize IO system */
+		fpgamgr_reprogram(buf, FPGA_BUFSIZ, 0);
 	}
 
 	/* If the IOSSM/full FPGA is already loaded, start DDR */
diff --git a/drivers/fpga/socfpga_arria10.c b/drivers/fpga/socfpga_arria10.c
index e7da705d1f..167368933c 100644
--- a/drivers/fpga/socfpga_arria10.c
+++ b/drivers/fpga/socfpga_arria10.c
@@ -26,12 +26,23 @@
 #define DEFAULT_DDR_LOAD_ADDRESS	0x400
 
 DECLARE_GLOBAL_DATA_PTR;
+bool fpgamgr_mode __section(".data");
 
 static const struct socfpga_fpga_manager *fpga_manager_base =
 		(void *)SOCFPGA_FPGAMGRREGS_ADDRESS;
 
 static void fpgamgr_set_cd_ratio(unsigned long ratio);
 
+static bool fpgamgr_reprogram_get(void)
+{
+	return fpgamgr_mode;
+}
+
+static void fpgamgr_reprogram_set(bool mode)
+{
+	fpgamgr_mode = mode;
+}
+
 static uint32_t fpgamgr_get_msel(void)
 {
 	u32 reg;
@@ -598,7 +609,8 @@ static int first_loading_rbf_to_buffer(struct udevice *dev,
 
 			if (strstr(uname, "fpga-periph") &&
 				(!is_fpgamgr_early_user_mode() ||
-				is_fpgamgr_user_mode())) {
+				is_fpgamgr_user_mode() ||
+				(fpgamgr_reprogram_get() == true))) {
 				fpga_node_name = uname;
 				printf("FPGA: Start to program ");
 				printf("peripheral/full bitstream ...\n");
@@ -895,6 +907,20 @@ void fpgamgr_program(const void *buf, size_t bsize, u32 offset)
 
 	socfpga_loadfs(&fpga_fsinfo, buf, bsize, offset);
 }
+
+void fpgamgr_reprogram(const void *buf, size_t bsize, u32 offset)
+{
+	fpga_fs_info fpga_fsinfo;
+
+	fpga_fsinfo.filename = get_fpga_filename();
+
+	fpgamgr_reprogram_set(true);
+
+	socfpga_loadfs(&fpga_fsinfo, buf, bsize, offset);
+
+	fpgamgr_reprogram_set(false);
+
+}
 #endif
 
 /* This function is used to load the core bitstream from the OCRAM. */
-- 
2.13.0

